"""Compute Mutation Rate

Usage:
  ComputeMutationRate.py <input1> <input2>


Options:
  -h --help               Show this message.
"""
from docopt import docopt
from Bio import SeqIO
import itertools


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'

    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''

def main(args):
    in1 = args['<input1>']
    in2 = args['<input2>']

    seq1 = SeqIO.parse(in1, 'fasta')
    seq2 = SeqIO.parse(in2, 'fasta')
    nMut = 0
    totLen = 0
    i = 0

    for s1, s2 in itertools.izip(seq1, seq2):
        o1 = ""
        o2 = ""
        newMut = True
        assert(s1.name == s2.name)
        totLen += len(s1.seq)
        for a,b in itertools.izip(s1.seq, s2.seq):
            if a != b:
                nMut+= 1
                newMut = True
                o1 += bcolors.OKGREEN + a + bcolors.ENDC
                o2 += bcolors.FAIL + b + bcolors.ENDC
            else:
                if newMut:
                    o1 += "***"
                    o2 += "***"
                    newMut = False

        i += 1
        print(o1)
        print(o2)
        if i % 1000 == 0:
            print("There were {} mutations in {} bases; total rate = {:0.2f}\r\r".format(\
                   nMut, totLen, (100.0 * nMut) / totLen))
    print("There were {} mutations in {} bases; total rate = {:0.2f}\r\r".format(\
           nMut, totLen, (100.0 * nMut) / totLen))

if __name__ == "__main__":
    args = docopt(__doc__, version="Compute Mutation Rate v1.0")
    main(args)



"""Mutate Reference.

Usage:
  MutateReference.py --in=<input> --out=<output> --rate=<rate>

Options:
  -h --help               Show this message.
  --in=<input>            The input reference sequence.
  --out=<output>          The output [mutated] sequences.
  --rate=<rate>           The rate of mutation.
"""
from docopt import docopt
from Bio import SeqIO
import random
import copy
import math
import sys

mutList = {'A' : ['T', 'C', 'G'],
           'T' : ['A', 'C', 'G'],
           'C' : ['A', 'G', 'T'],
           'G' : ['A', 'C', 'T'],
           'a' : ['c', 'g', 't'],
           't' : ['a', 'c', 'g'],
           'c' : ['a', 'g', 't'],
           'g' : ['a', 'c', 't'],
           'N' : ['A', 'T', 'C', 'G'],
           'n' : ['a', 't', 'c', 'g']}

def mutate(base):
    ml = mutList[base]
    return ml[random.randint(0, len(ml)-1)]

def randCeilFloor(x):
    if x < 1.0 or random.random() < 0.5:
        return int(math.ceil(x))
    else:
        return int(math.floor(x))

def main(args):
    mutRate = float(args['--rate'])
    refIn = args['--in']
    refOut = args['--out']

    ofile = open(refOut,'a')
    seqs = SeqIO.parse(refIn, 'fasta')
    currBase = 0
    nMut = 0
    totLen = 0
    nextMut = currBase + randCeilFloor(random.expovariate(mutRate))
    for i, s in enumerate(seqs):
        lenS = len(s.seq)
        firstBase = currBase
        lastBase = currBase + lenS
        totLen += lenS
        o = copy.copy(s.seq)
        x = s.seq.tomutable()
        while nextMut < lastBase:
            offset = nextMut - firstBase
            orig = o[offset]
            x[offset] = mutate(orig)
            assert(x[offset] != orig)
            nMut += 1
            currBase = nextMut
            nextMut = currBase + randCeilFloor(random.expovariate(mutRate))
        currBase = lastBase
        #assert(len(o) == len(x))
        #for a,b in itertools.izip(o,x):
        #    if a != b: nMut += 1
        s.seq = x
        SeqIO.write(s, ofile, 'fasta')
        if (i % 1000 == 0):
            sys.stderr.write("processed {} records; performed {} mutations; rate = {:.2f}%\r\r".format(i, nMut, (100.0 * nMut) / totLen))
        # print("\n")/tmp/hg18_transcripts_mut0.001.fa/tmp/hg18_transcripts_mut0.001.fa/tmp/hg18_transcripts_mut0.001.fa
        # print('Num Rec = {}, Num Mut = {}, Tot. length = {}, rate = {}'.format(i, nMut, totLen, (100.0 * nMut) / totLen))
    print("\ndone.")

if __name__ == "__main__":
    arguments = docopt(__doc__, version="Mutate Reference 1.0")
    main(arguments)



import argparse
import errno    
import os


def mkdir_p(path):
    """
    http://stackoverflow.com/questions/600268/mkdir-p-functionality-in-python
    """
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

def computeSimMetrics(args):
    import pandas as pd
    import json
    simFile = args.sim
    estFile = args.est
    sd = pd.read_table(simFile).set_index('Name')
    ed = pd.read_table(estFile).set_index('Name')
    d = sd.join(ed, rsuffix='_est')

    res = {}
    res['pearson'] = d['TPM'].corr(d['TPM_est'])
    res['spearman'] = d['TPM'].corr(d['TPM_est'], method='spearman')

    import os.path
    pdir = os.path.dirname(args.out)
    if not os.path.exists(pdir):
        mkdir_p(pdir)

    with open(args.out, 'w') as outfile:
        json.dump(res, outfile)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Test simulated accuracy')
    parser.add_argument('--sim', type=str, help='path to simulated tpm')
    parser.add_argument('--est', type=str, help='path to estimated tpm')
    parser.add_argument('--out', type=str, help='where to write the output')
    args = parser.parse_args()
    computeSimMetrics(args)





import gzip
import struct
import argparse
import os
import logging
import logging.handlers
import sys
import errno
import json

# from: http://stackoverflow.com/questions/600268/mkdir-p-functionality-in-python
def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

def main(args):
    logging.basicConfig(level=logging.INFO)
    quantDir = args.quantDir
    auxDir = "aux"

    # Check for a custom auxDir
    with open(os.path.sep.join([quantDir, "cmd_info.json"])) as cmdFile:
        dat = json.load(cmdFile)
        if 'auxDir' in dat:
            auxDir = dat['auxDir']

    bootstrapFile = os.path.sep.join([quantDir, auxDir, "bootstrap", "bootstraps.gz"])
    nameFile = os.path.sep.join([quantDir, auxDir, "bootstrap", "names.tsv.gz"])
    if not os.path.isfile(bootstrapFile):
       logging.error("The required bootstrap file {} doesn't appear to exist".format(bootstrapFile)) 
       sys.exit(1)
    if not os.path.isfile(nameFile):
       logging.error("The required transcript name file {} doesn't appear to exist".format(nameFile)) 
       sys.exit(1)
    
    txpNames = None
    with gzip.open(nameFile) as nf:
        txpNames = nf.read().decode().strip().split('\t')
    
    ntxp = len(txpNames)
    logging.info("Expecting bootstrap info for {} transcripts".format(ntxp))
    
    with open(os.path.sep.join([quantDir, auxDir, "meta_info.json"])) as fh:
        meta_info = json.load(fh)
        
    if meta_info['samp_type'] == 'gibbs':
        #s = struct.Struct('<' + 'i' * ntxp)
        s = struct.Struct('@' + 'd' * ntxp)
    elif meta_info['samp_type'] == 'bootstrap':
        s = struct.Struct('@' + 'd' * ntxp)
    else:
        logging.error("Unknown sampling method: {}".format(meta_info['samp_type']))
        sys.exit(1)
        
    numBoot = 0
    outDir = args.outDir
    if os.path.exists(outDir):
        if os.path.isfile(outDir):
            logging.error("The requested output directory {} already exists, but is a file".format(outDir))
            sys.exit(1)
        else:
            logging.warn("The requested output directory {} already exists --- any existing bootstraps may be overwritten".format(outDir))
    else:
        mkdir_p(outDir)
    
    outFile = os.path.sep.join([outDir, 'quant_bootstraps.tsv'])
    with open(outFile,'w') as ofile:
        # write the header
        ofile.write('\t'.join(txpNames) + '\n')
        
        # Now, iterate over the bootstrap samples and write each
        with gzip.open(bootstrapFile) as bf:
            while True:
                try:
                    x = s.unpack_from(bf.read(s.size))
                    xs = map(str, x)
                    ofile.write('\t'.join(xs) + '\n')
                    numBoot += 1
                except Exception as ex:
                    logging.info("read all bootstrap values")
                    break
                except BaseException:
                    logging.info("read all bootstrap values")
                    break

    logging.info("wrote {} bootstrap samples".format(numBoot))
    logging.info("converted bootstraps successfully.")

if __name__ == "__main__":
   parser = argparse.ArgumentParser(description="Convert bootstrap results to text format") 
   parser.add_argument('quantDir', type=str, help="path to (sailfish / salmon) quantification directory")
   parser.add_argument('outDir', type=str, help="path to directory where results should be written")
   args = parser.parse_args()
   main(args)
